#include <allegro5/allegro.h>
#include <allegro5/allegro_font.h>
#include <allegro5/allegro_ttf.h>
#include <allegro5/allegro_primitives.h>
#include <allegro5/allegro_audio.h>
#include <math.h>

#include "main.h"
#include "game.h"
#include "input.h"
#include "resscale.h"
#include "rng.h"
#include "dialog.h"
#include "particle.h"

// State variables (this is what is saved)
int timed = 0, // 0-3600 (One minute)
	stage = 0, // 1-4, 0 being new chip dialog, the others being the dialog or fireworks
	wins = 0,
	loses = 0;
float heat = 0,
	topSpeed = 0,
	thirst = 255,
	hunger = 255,
	health = 255;

void load_game(void) {
	ALLEGRO_PATH *path;
	ALLEGRO_FILE *progress;
	ALLEGRO_STATE state;
	char cstr[100];
	const char *file;

	al_store_state(&state, ALLEGRO_STATE_NEW_FILE_INTERFACE);
	al_set_standard_file_interface();
	path = al_get_standard_path(ALLEGRO_USER_SETTINGS_PATH);
	al_drop_path_tail(path);
	al_set_path_filename(path, "cfprogress.dat");
	file = al_path_cstr(path, ALLEGRO_NATIVE_PATH_SEP);

	progress = al_fopen(file, "rb");
	if (progress) {
		al_fgets(progress, cstr, 99);
		stage = atoi(cstr);
		al_fgets(progress, cstr, 99);
		timed = atoi(cstr);
		al_fgets(progress, cstr, 99);
		heat = atoi(cstr);
		al_fgets(progress, cstr, 99);
		topSpeed = atoi(cstr);
		al_fgets(progress, cstr, 99);
		thirst = atoi(cstr);
		al_fgets(progress, cstr, 99);
		hunger = atoi(cstr);
		al_fgets(progress, cstr, 99);
		health = atoi(cstr);
		al_fgets(progress, cstr, 99);
		wins = atoi(cstr);
		al_fgets(progress, cstr, 99);
		loses = atoi(cstr);
		al_fclose(progress);
	}
	else {
		stage = 0;
		timed = 0;
		heat = 0;
		topSpeed = 0;
		thirst = 255;
		hunger = 255;
		health = 255;
		wins = 0;
		loses = 0;
	}

	if (path) al_destroy_path(path);
	al_restore_state(&state);
}

void save_game(void) {
	ALLEGRO_PATH *path;
	ALLEGRO_FILE *progress;
	ALLEGRO_STATE state;
	const char *file;

	al_store_state(&state, ALLEGRO_STATE_NEW_FILE_INTERFACE);
	al_set_standard_file_interface();
	path = al_get_standard_path(ALLEGRO_USER_SETTINGS_PATH);
	al_drop_path_tail(path);
	al_set_path_filename(path, "cfprogress.dat");
	file = al_path_cstr(path, ALLEGRO_NATIVE_PATH_SEP);

	progress = al_fopen(file, "wb");
	if (progress) {
		al_fprintf(progress, "%d\n", stage);
		al_fprintf(progress, "%d\n", timed);
		al_fprintf(progress, "%f\n", heat);
		al_fprintf(progress, "%f\n", topSpeed);
		al_fprintf(progress, "%f\n", thirst);
		al_fprintf(progress, "%f\n", hunger);
		al_fprintf(progress, "%f\n", health);
		al_fprintf(progress, "%d\n", wins);
		al_fprintf(progress, "%d\n", loses);
		al_fclose(progress);
	}

	if (path) al_destroy_path(path);
	al_restore_state(&state);
}

void play_game(void) {
	// Variables
	ALLEGRO_EVENT event;
	ALLEGRO_TRANSFORM def, scaled;
	ALLEGRO_FONT *hud;
	ALLEGRO_COLOR white = al_map_rgb(255, 255, 255), red = al_map_rgb(255, 0, 0), yellow = al_map_rgb(255, 255, 0);
	ALLEGRO_BITMAP *map, *tiles,
		*chip,
		*tempIcon,
		*speedIcon,
		*thirstIcon,
		*hungerIcon,
		*healthIcon,
		*batteryIcon,
		*waterIcon,
		*functionIcon,
		*bucketIcon,
		*exitIcon;
	ALLEGRO_SAMPLE *cry;
	ALLEGRO_SAMPLE_ID cryID;

	ResScale scale(640, 480);
	Input input;
	rng r;
	Particle particles[1000];
	Dialog dialogs[] = {
		{
			"Well, I see you have your chip.", "Do well to raise it.", 0, 0,
			"Will do!", "Don't tell me want to do!", "Whatever", "I'll try.",
			-1, 1, 1, 2
		},
		{
			"With an attitude like that...", 0, 0, 0,
			"CLose", 0, 0, 0,
			-1, -1, -1, -1
		},
		{
			"As long as you you give it your best!", 0, 0, 0,
			"Close", 0, 0, 0,
			-1, -1, -1, -1
		},
		{
			"Empty Dialog", "Sorry to", "waste your", "time",
			"WTF?", "OMG!", "BBQ", "WUT?",
			4, 5, 6, 7
		},
		{
			"Code created during SHpaecekd", "can be a major WTF", 0, 0,
			"...", "WUT?", 0, 0,
			-1, 7, -1, -1
		},
		{
			"Such indeed", 0, 0, 0,
			"Close", 0, 0, 0,
			-1, -1, -1, -1
		},
		{
			"BBQ is yummy!", 0, 0, 0,
			"Close", 0, 0, 0,
			-1, -1, -1, -1
		},
		{
			"WUT indeed", 0, 0, 0,
			"Close", 0, 0, 0,
			-1, -1, -1, -1
		},
		{ // 8
			"Sorry about the particles", "Couldn't think of a better way",  "to add them with the short time I had", 0,
			"I hate you...", "Should of had flaming water", "Whatever", "Will there be more?",
			9, 10, -1, 11
		},
		{
			"You could stop playing", 0, 0, "the game",
			"Close", 0, 0, 0,
			-1, -1, -1, -1
		},
		{
			"I wanted to!", "Then when the water burnt down,", "There would be particle based smoke!", 0,
			"Close", 0, 0, 0,
			-1, -1, -1, -1
		},
		{
			"Oh yes!", 0, 0, 0,
			"Bring it!", 0, 0, 0,
			-1, -1, -1, -1
		},
		{ //12
			"You killed your chip!", "Try again!", 0, 0,
			"Ok", "No!", 0, 0,
			-1, 13, -1, -1
		},
		{
			"Well too bad!", "Unless you quit the game", "You'll have to always be raising a chip", 0,
			"Alright, whatever", "Bite me!", "...", 0,
			1, 1, -1, -1
		},
		{ // 14
			"Well done, you did a great job", "Raise another fine chip", 0, 0,
			"Yes sir!", "Whatever", 0, 0,
			-1, 1, 0, 0
		},
		{ // 15
			"Alright, that's enough chips for now.", "Take a break", 0, "See you next time... maybe",
			"Quit", 0, 0, 0,
			-2, -1, -1, -1
		},
		{ // 16
			"Alright, you're too much of a killer", "Get out of here!", 0, "Come back only if you care",
			"BYE!", 0, 0, 0,
			-2, -1, -1, -1
		}
	};
	Dialog *cDialog = NULL;

	int i = 0, x = 0, y = 0,
		chipX = 320, chipY = 240, chipTX = 320, chipTY = 240, chipT = 0, chipZ = 0, chipDZ = 0, chipA = 0, chipPA = 20,
		metalX = 213, metalY = 338,
		width = al_get_display_width(al_get_current_display()),
		height = al_get_display_height(al_get_current_display()),
		swidth = (int)(width / (float)height * 480),
		state = 0, // 0 for unknown (Need to see stage), 1 for field, 2 for dialog
		nextstate = 0, // 0 for current state, otherwise, what state will change to
		transCount = 0, // 0-127
		selectedButton = 0, // 0-4 (0 exit, 1 is feed, 2 is water, 3 is func, 4 is bucket)
		currentDialog = 0,
		nextDialog = 0,
		firstDialog = 3,
		particleLevel = 0,
		particleShot = 0;
	bool running = true,
		redraw = true,
		trans = false,
		drawButtons = true,
		crying = false;
	float fval = 0.0;

	// Load files
	hud = al_load_font("VeraMoBd.ttf", -20, ALLEGRO_TTF_NO_AUTOHINT);

	map = al_load_bitmap("map.jpg");
	tiles = al_load_bitmap("tiles.png");
	tempIcon = al_create_sub_bitmap(tiles, 0, 0, 16, 16);
	speedIcon = al_create_sub_bitmap(tiles, 0, 16, 16, 16);
	thirstIcon = al_create_sub_bitmap(tiles, 0, 32, 16, 16);
	hungerIcon = al_create_sub_bitmap(tiles, 0, 48, 16, 16);
	healthIcon = al_create_sub_bitmap(tiles, 0, 64, 16, 16);
	exitIcon = al_create_sub_bitmap(tiles, 0, 80, 16, 16);
	batteryIcon = al_create_sub_bitmap(tiles, 16, 0, 135, 135);
	waterIcon = al_create_sub_bitmap(tiles, 16, 135, 135, 135);
	functionIcon = al_create_sub_bitmap(tiles, 151, 0, 135, 135);
	bucketIcon = al_create_sub_bitmap(tiles, 151, 135, 135, 135);
	chip = al_create_sub_bitmap(tiles, 299, 15, 86, 53);
	cry = al_load_sample("cry.ogg");

	// Set the variables that haven't been set yet
	al_identity_transform(&def);
	scale.getTransform(&scaled);

	load_game();
	al_start_timer(timer);
	while (running) {
		al_wait_for_event(queue, NULL);
		while (!al_event_queue_is_empty(queue)) {
			al_get_next_event(queue, &event);

			switch (event.type) {
			case ALLEGRO_EVENT_KEY_DOWN: {
				input.keyDown(&event);
				break;
			}
			case ALLEGRO_EVENT_KEY_UP: {
				input.keyUp(&event);
				break;
			}
			case ALLEGRO_EVENT_MOUSE_AXES: {
				input.mouseMove(&event);
				break;
			}
			case ALLEGRO_EVENT_MOUSE_BUTTON_DOWN: {
				input.mouseButDown(&event);
				break;
			}
			case ALLEGRO_EVENT_MOUSE_BUTTON_UP: {
				input.mouseButDown(&event);
				break;
			}
			case ALLEGRO_EVENT_DISPLAY_CLOSE: {
				running = false;
				break;
			}
			case ALLEGRO_EVENT_TIMER: {
				if (input.pausePressed()) {
					// Pause menu?
					running = false;
				}

				if (!trans) {
					switch (state) {
					case 0: {
						if (stage == 0) {
							if (timed == 0 && stage == 0) {
								state = 2;
								currentDialog = nextDialog = 0;
							}
							else {
								state = 1;
							}
						}
						else {
							if (stage == 2 || stage == 4) {
								particleLevel = stage / 2 * 500;
							}
							state = 1;
							currentDialog = nextDialog = firstDialog;
						}
						redraw = false;
						break;
					}
					case 1: {
						// Button handling
						selectedButton = -1;
						x = input.mouseX();
						y = input.mouseY();

						if (drawButtons) {
							if (x > (width - 290) && x < (width - 155)) {
								if (y > 140 && y < 275) {
									selectedButton = 1;
								}
								else if (y > 285 && y < 420) {
									selectedButton = 3;
								}
							}
							else if (x >(width - 145) && x < (width - 10)) {
								if (y > 140 && y < 275) {
									selectedButton = 2;
								}
								else if (y > 285 && y < 420) {
									selectedButton = 4;
								}
							}

							// Exit button
							if (x >(width - 26) && x < (width - 10) && y >(height - 26) && y < (height - 10)) {
								selectedButton = 0;
							}
							if (selectedButton == 0 && input.mouseButtonPressed(0)) {
								running = false;
							}
						}

						// Update the stats on chip, and move him around. Control what the user is doing, etc check to see if the dialog needs to appear
						if (topSpeed > 0) {
							// Calculate the new heat
							if (heat < topSpeed) {
								heat += 0.25;
							}
							else {
								if (heat) heat -= 0.125;
							}
						}
						else {
							if (heat) heat -= 0.125;
						}
						hunger -= 0.25;
						if (hunger < 0) {
							health -= 0.25;
							hunger = 0;
						}
						thirst -= 0.25;
						if (thirst < 0) {
							health -= 0.25;
							thirst = 0;
						}
						if (heat > 200) {
							health -= 0.25;
						}
						if (heat > 255) {
							heat = 255;
						}
						if (health < 100 && !crying) {
							al_play_sample(cry, 1.0, 0.0, 1.0, ALLEGRO_PLAYMODE_LOOP, &cryID);
							crying = true;
						}
						if (health > 100 && crying) {
							al_stop_sample(&cryID);
							crying = false;
						}
						if (hunger > 200 && thirst > 200 && health < 255) {
							hunger -= 5;
							thirst -= 5;
							health += 10;
						}
						if (health < 1) {
							nextstate = 2;
							nextDialog = 12;
							trans = true;
							heat = 0;
							topSpeed = 0;
							thirst = 255;
							hunger = 255;
							health = 255;
							stage = 0;
							timed = 0;
							if (crying) al_stop_sample(&cryID);
							crying = false;
							loses++;
							if (loses - 27 > wins) {
								nextDialog = 16;
							}
						}
						if (topSpeed > 254 && health > 254 && hunger > 200 && thirst > 200) {
							nextstate = 2;
							nextDialog = 14;
							trans = true;
							heat = 0;
							topSpeed = 0;
							thirst = 255;
							hunger = 255;
							health = 255;
							stage = 0;
							timed = 0;
							if (crying) al_stop_sample(&cryID);
							crying = false;
							wins++;
							if (wins - 10 > loses) {
								nextDialog = 15;
							}
						}
						if (selectedButton > 0 && input.mouseButtonPressed(0)) {
							if (selectedButton == 1 && hunger < 100) {
								hunger = 255;
								chipT = -20;
								drawButtons = false;
							}
							if (selectedButton == 2 && thirst < 100) {
								thirst = 255;
								heat -= 100;
								if (heat < 0) {
									heat = 0;
								}
								chipT = -30;
								drawButtons = false;
							}
							if (selectedButton == 3) {
								topSpeed += 20;
								if (topSpeed > 255) {
									topSpeed = 255;
								}
								chipT = -50;
								drawButtons = false;
							}
							if (selectedButton == 4) {
								heat = 0;
								topSpeed /= 4;
								chipT = -50;
								drawButtons = false;
							}
						}

						if (chipT == 0) {
							// Chip wants to move to a new target
							chipTX = r.rand() % 572;
							chipTY = r.rand() % 427;
							chipT = (r.rand() % 60) + 120;
							drawButtons = true;
						}
						else if (chipT > 0) {
							chipT--;
							if (chipX > chipTX) {
								chipX--;
							}
							if (chipX < chipTX) {
								chipX++;
							}
							if (chipY > chipTY) {
								chipY--;
							}
							if (chipY < chipTY) {
								chipY++;
							}
						}
						else if (chipT < 0) {
							// Chip is acting on something the user did
							chipT++;
						}
						// Bounce!
						if (chipX < 213 && chipY > 285) {
							chipA = chipPA;
						}
						else {
							chipA = 0;
						}
						if (chipZ < chipDZ) {
							chipZ++;
						}
						if (chipZ > chipDZ) {
							chipZ--;
						}
						if (chipDZ && chipZ == chipDZ) {
							chipDZ = 0;
						}
						if (chipDZ == 0 && chipZ == 0 && chipA) {
							chipDZ = chipA;
						}

						if (particleLevel) {
							for (i = 0; i < particleLevel; i++) {
								particles[i].logic();
								if (!particles[i].isActive() && particleShot < 20) {
									particleShot++;
									particles[i].init(Vector2(chipX + r.rand() % 68, chipY - (float)chipZ), Vector2((r.rand() %  30 - 15.0) * 0.25, (r.rand() % 30 - 15.0) * 0.25), r.rand() % 60 + 60);
								}
							}
							particleShot = 0;
						}

						// Update the clock
						timed++;
						timed = timed % 3600;
						if (timed == 0) {
							stage++;
							stage = stage % 5;
							if (!stage) {
								stage = 1;
							}
							if (stage == 1) {
								trans = true;
								nextstate = 2;
								nextDialog = 3;
							}
							if (stage == 3) {
								trans = true;
								nextstate = 2;
								nextDialog = 8;
							}
							if (stage == 2 || stage == 4) {
								particleLevel = stage / 2 * 500;
							}
							else {
								particleLevel = 0;
								for (i = 0; i < 1000; i++) {
									particles[i].kill();
								}
							}
						}
						break;
					}
					case 2: {
						// Dialog handling code.
						selectedButton = 0;
						x = input.mouseX();
						y = input.mouseY();
						cDialog = &dialogs[currentDialog];
						if (cDialog->button4 && cDialog->button3 && cDialog->button2) {
							// Four button handling
							if (x > 1 && x < width / 2 - 1 && y > height - 65 && y < height - 36) {
								selectedButton = 1;
							}
							if (x > width / 2 + 1 && x < width - 1 && y > height - 65 && y < height - 36) {
								selectedButton = 2;
							}
							if (x > 1 && x < width / 2 - 1 && y > height - 34 && y < height - 4) {
								selectedButton = 3;
							}
							if (x > width / 2 + 1 && x < width - 1 && y > height - 34 && y < height - 4) {
								selectedButton = 4;
							}
						}
						else if (cDialog->button3 && cDialog->button2) {
							// Three button handling
							if (x > 1 && x < width / 2 - 1 && y > height - 65 && y < height - 36) {
								selectedButton = 1;
							}
							if (x > width / 2 + 1 && x < width - 1 && y > height - 65 && y < height - 36) {
								selectedButton = 2;
							}
							if (x > 1 && x < width - 1 && y > height - 34 && y < height - 4) {
								selectedButton = 3;
							}
						}
						else if (cDialog->button2) {
							// Two button handling
							if (x > 1 && x < width / 2 - 1 && y > height - 34 && y < height - 4) {
								selectedButton = 1;
							}
							if (x > width / 2 + 1 && x < width - 1 && y > height - 34 && y < height - 4) {
								selectedButton = 2;
							}
						}
						else {
							// One button handling
							if (x > 1 && x < width - 1 && y > height - 34 && y < height - 4) {
								selectedButton = 1;
							}
						}

						if (input.mouseButtonPressed(0)) {
							switch (selectedButton) {
							case 1: {
								if (cDialog->next1 != -1) {
									nextstate = 2;
									nextDialog = cDialog->next1;
								}
								else {
									nextstate = 1;
								}
								trans = true;
								selectedButton = -1;
								break;
							}
							case 2: {
								if (cDialog->next2 != -1) {
									nextstate = 2;
									nextDialog = cDialog->next2;
								}
								else {
									nextstate = 1;
								}
								trans = true;
								selectedButton = -1;
								break;
							}
							case 3: {
								if (cDialog->next3 != -1) {
									nextstate = 2;
									nextDialog = cDialog->next3;
								}
								else {
									nextstate = 1;
								}
								trans = true;
								selectedButton = -1;
								break;
							}
							case 4: {
								if (cDialog->next4 != -1) {
									nextstate = 2;
									nextDialog = cDialog->next4;
								}
								else {
									nextstate = 1;
								}
								trans = true;
								selectedButton = -1;
								break;
							}
							}
						}
						break;
					}
					}
				}
				else {
					if (nextstate) {
						transCount++;
						transCount++;
						transCount++;
						transCount++;
						if (transCount > 127) {
							state = nextstate;
							transCount = 127;
							nextstate = 0;
							currentDialog = nextDialog;
							nextDialog = 0;
							chipT = 0;
							if (currentDialog == -2) {
								heat = 0;
								topSpeed = 0;
								thirst = 255;
								hunger = 255;
								health = 255;
								wins = 0;
								loses = 0;
								running = false;
								redraw = false;
								state = 0;
							}
						}
					}
					else {
						trans = false;
						transCount = 0;
					}
				}
				
				input.resetChanged();
			}
			case ALLEGRO_EVENT_DISPLAY_EXPOSE:
				redraw = true;
				break;
			}
		}

		if (redraw) {
			al_clear_to_color(al_map_rgb(0, 0, 0));
			switch (state) {
			case 1: {
				// Draw chip and his world
				al_use_transform(&scaled);
				al_draw_bitmap(map, 0, 0, 0);
				al_draw_filled_ellipse(chipX + 43, chipY + 53, 43.0 * ((chipPA - (chipZ / 2)) / (float)chipPA), 12.0 * ((chipPA - (chipZ / 2)) / (float)chipPA), al_map_rgba(0, 0, 0, 150));
				for (i = 0; i < particleLevel; i++) {
					particles[i].draw();
				}
				al_draw_bitmap(chip, chipX, chipY - chipZ, 0);

				// Draw the hud
				al_use_transform(&def);
				al_draw_filled_rectangle(width - 300, 0, width, height, al_map_rgba(255, 127, 127, 127));

				// Temp
				al_draw_bitmap(tempIcon, width - 290, 10, 0);
				al_draw_filled_rectangle(width - 266, 10, width - 10, 26, al_map_rgb(0, 0, 0));
				al_draw_filled_rectangle(width - 266, 10, width - (10 + (255 - heat)), 26, al_map_rgb(255, 0, 0));

				// topSpeed
				al_draw_bitmap(speedIcon, width - 290, 36, 0);
				al_draw_filled_rectangle(width - 266, 36, width - 10, 52, al_map_rgb(0, 0, 0));
				al_draw_filled_rectangle(width - 266, 36, width - (10 + (255 - topSpeed)), 52, al_map_rgb(255, 0, 0));

				// hunger
				al_draw_bitmap(hungerIcon, width - 290, 62, 0);
				al_draw_filled_rectangle(width - 266, 62, width - 10, 78, al_map_rgb(0, 0, 0));
				al_draw_filled_rectangle(width - 266, 62, width - (10 + (255 - hunger)), 78, al_map_rgb(255, 0, 0));

				// thirst
				al_draw_bitmap(thirstIcon, width - 290, 88, 0);
				al_draw_filled_rectangle(width - 266, 88, width - 10, 104, al_map_rgb(0, 0, 0));
				al_draw_filled_rectangle(width - 266, 88, width - (10 + (255 - thirst)), 104, al_map_rgb(255, 0, 0));

				// health
				al_draw_bitmap(healthIcon, width - 290, 114, 0);
				al_draw_filled_rectangle(width - 266, 114, width - 10, 130, al_map_rgb(0, 0, 0));
				al_draw_filled_rectangle(width - 266, 114, width - (10 + (255 - health)), 130, al_map_rgb(255, 0, 0));

				if (drawButtons) {
					// Interactive buttons
					if (selectedButton == 1) {
						al_draw_filled_rectangle(width - 290, 140, width - 155, 275, al_map_rgb(255, 255, 255));
					}
					al_draw_bitmap(batteryIcon, width - 290, 140, 0);

					if (selectedButton == 2) {
						al_draw_filled_rectangle(width - 145, 140, width - 10, 275, al_map_rgb(255, 255, 255));
					}
					al_draw_bitmap(waterIcon, width - 145, 140, 0);

					if (selectedButton == 3) {
						al_draw_filled_rectangle(width - 290, 285, width - 155, 420, al_map_rgb(255, 255, 255));
					}
					al_draw_bitmap(functionIcon, width - 290, 285, 0);

					if (selectedButton == 4) {
						al_draw_filled_rectangle(width - 145, 285, width - 10, 420, al_map_rgb(255, 255, 255));
					}
					al_draw_bitmap(bucketIcon, width - 145, 285, 0);

					if (selectedButton == 0) {
						al_draw_filled_rectangle(width - 26, height - 26, width - 10, height - 10, al_map_rgb(255, 255, 255));
					}
					al_draw_bitmap(exitIcon, width - 26, height - 26, 0);
				}

				al_draw_filled_rectangle(0, 0, 170, 50, al_map_rgba(255, 127, 127, 255));
				al_draw_textf(hud, al_map_rgb(255, 255, 255), 5, 5, 0, "Good Chips: %d", wins);
				al_draw_textf(hud, al_map_rgb(255, 0, 0), 5, 25, 0, "Dead Chips: %d", loses);
				break;
			}
			case 2: {
				// Message to user
				cDialog = &dialogs[currentDialog];
				al_draw_text(hud, al_map_rgb(255, 255, 255), width / 2, height / 2 - 60, ALLEGRO_ALIGN_CENTER, cDialog->message1);
				if (cDialog->message2) {
					al_draw_text(hud, al_map_rgb(255, 255, 255), width / 2, height / 2 - 30, ALLEGRO_ALIGN_CENTER, cDialog->message2);
				}
				if (cDialog->message3) {
					al_draw_text(hud, al_map_rgb(255, 255, 255), width / 2, height / 2, ALLEGRO_ALIGN_CENTER, cDialog->message3);
				}
				if (cDialog->message4) {
					al_draw_text(hud, al_map_rgb(255, 255, 255), width / 2, height / 2 + 30, ALLEGRO_ALIGN_CENTER, cDialog->message4);
				}

				if (cDialog->button4 && cDialog->button3 && cDialog->button2) {
					// Four button layout

					al_draw_filled_rounded_rectangle(1, height - 65, width / 2 - 1, height - 36, 3, 3, al_map_rgb(127, 127, 127));
					if (selectedButton == 1) {
						al_draw_rounded_rectangle(1, height - 65, width / 2, height - 36, 3, 3, al_map_rgb(255, 255, 255), 1);
					}
					al_draw_text(hud, al_map_rgb(255, 255, 255), width / 4, height - 60, ALLEGRO_ALIGN_CENTER, cDialog->button1);

					al_draw_filled_rounded_rectangle(width / 2 + 1, height - 65, width - 1, height - 36, 3, 3, al_map_rgb(127, 127, 127));
					if (selectedButton == 2) {
						al_draw_rounded_rectangle(width / 2 + 1, height - 65, width - 1, height - 36, 3, 3, al_map_rgb(255, 255, 255), 1);
					}
					al_draw_text(hud, al_map_rgb(255, 255, 255), width / 4 + width / 2, height - 60, ALLEGRO_ALIGN_CENTER, cDialog->button2);

					al_draw_filled_rounded_rectangle(1, height - 34, width / 2 - 1, height - 4, 3, 3, al_map_rgb(127, 127, 127));
					if (selectedButton == 3) {
						al_draw_rounded_rectangle(1, height - 34, width / 2 - 1, height - 4, 3, 3, al_map_rgb(255, 255, 255), 1);
					}
					al_draw_text(hud, al_map_rgb(255, 255, 255), width / 4, height - 30, ALLEGRO_ALIGN_CENTER, cDialog->button3);

					al_draw_filled_rounded_rectangle(width / 2 + 1, height - 34, width - 1, height - 4, 3, 3, al_map_rgb(127, 127, 127));
					if (selectedButton == 4) {
						al_draw_rounded_rectangle(width / 2 + 1, height - 34, width - 1, height - 4, 3, 3, al_map_rgb(255, 255, 255), 1);
					}
					al_draw_text(hud, al_map_rgb(255, 255, 255), width / 4 + width / 2, height - 30, ALLEGRO_ALIGN_CENTER, cDialog->button4);

				}
				else if (cDialog->button3 && cDialog->button2) {
					// Three button layout

					al_draw_filled_rounded_rectangle(1, height - 65, width / 2 - 1, height - 36, 3, 3, al_map_rgb(127, 127, 127));
					if (selectedButton == 1) {
						al_draw_rounded_rectangle(0, height - 65, width / 2, height - 36, 3, 3, al_map_rgb(255, 255, 255), 1);
					}
					al_draw_text(hud, al_map_rgb(255, 255, 255), width / 4, height - 60, ALLEGRO_ALIGN_CENTER, cDialog->button1);

					al_draw_filled_rounded_rectangle(width / 2 + 1, height - 65, width - 1, height - 36, 3, 3, al_map_rgb(127, 127, 127));
					if (selectedButton == 2) {
						al_draw_rounded_rectangle(width / 2 + 1, height - 65, width - 1, height - 36, 3, 3, al_map_rgb(255, 255, 255), 1);
					}
					al_draw_text(hud, al_map_rgb(255, 255, 255), width / 4 + width / 2, height - 60, ALLEGRO_ALIGN_CENTER, cDialog->button2);

					al_draw_filled_rounded_rectangle(1, height - 34, width - 1, height - 4, 3, 3, al_map_rgb(127, 127, 127));
					if (selectedButton == 3) {
						al_draw_rounded_rectangle(1, height - 34, width - 1, height - 4, 3, 3, al_map_rgb(255, 255, 255), 1);
					}
					al_draw_text(hud, al_map_rgb(255, 255, 255), width / 2, height - 30, ALLEGRO_ALIGN_CENTER, cDialog->button3);

				}
				else if (cDialog->button2) {
					// Two button layout

					al_draw_filled_rounded_rectangle(1, height - 34, width / 2 - 1, height - 4, 3, 3, al_map_rgb(127, 127, 127));
					if (selectedButton == 1) {
						al_draw_rounded_rectangle(0, height - 34, width / 2, height - 4, 3, 3, al_map_rgb(255, 255, 255), 1);
					}
					al_draw_text(hud, al_map_rgb(255, 255, 255), width / 4, height - 30, ALLEGRO_ALIGN_CENTER, cDialog->button1);

					al_draw_filled_rounded_rectangle(width / 2 + 1, height - 34, width - 1, height - 4, 3, 3, al_map_rgb(127, 127, 127));
					if (selectedButton == 2) {
						al_draw_rounded_rectangle(width / 2 + 1, height - 34, width - 1, height - 4, 3, 3, al_map_rgb(255, 255, 255), 1);
					}
					al_draw_text(hud, al_map_rgb(255, 255, 255), width / 4 + width / 2, height - 30, ALLEGRO_ALIGN_CENTER, cDialog->button2);

				}
				else {
					// One button layout

					al_draw_filled_rounded_rectangle(1, height - 34, width - 1, height - 4, 3, 3, al_map_rgb(127, 127, 127));
					if (selectedButton == 1) {
						al_draw_rounded_rectangle(1, height - 34, width - 1, height - 4, 3, 3, al_map_rgb(255, 255, 255), 1);
					}
					al_draw_text(hud, al_map_rgb(255, 255, 255), width / 2, height - 30, ALLEGRO_ALIGN_CENTER, cDialog->button1);

				}
				break;
			}
			}

			// Fade box
			al_use_transform(&def);
			fval = transCount / 127.0;
			if (fval > 1.0) {
				fval = 1.0;
			}
			al_draw_filled_rectangle(0, 0, width, height, al_map_rgba_f(0, 0, 0, fval));

			al_flip_display();
			redraw = false;
		}
	}

	// Clean up
	save_game();
	al_stop_timer(timer);
	al_flush_event_queue(queue);
}
